/* 
   circles - calculate identified circles statistics from CMB data (cosmic topology)

   Copyright (C) 2004 Boud Roukema

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

#include <termios.h>
#include <grp.h>
#include <pwd.h>
*/

#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include "system.h"


#ifdef HAVE_ARGP_H
#  include <argp.h>
#endif  /* HAVE_ARGP_H */

#include <stdlib.h>


#define EXIT_FAILURE 1

#if ENABLE_NLS
# include <libintl.h>
# define _(Text) gettext (Text)
#else
# define textdomain(Domain)
# define _(Text) Text
#endif
#define N_(Text) Text


#include "lib/circles.h" 


char *xmalloc ();
char *xrealloc ();
char *xstrdup ();

#ifdef HAVE_ARGP_H
  static error_t parse_opt (int key, char *arg, struct argp_state *state);
  static void show_version (FILE *stream, struct argp_state *state);
#else
  static void show_version (FILE *stream);
#endif


/* argp option keys */
enum {DUMMY_KEY=129
      ,NOWARN_KEY='w'
      ,DIRECTORY_KEY='d'
};

/* Option flags and variables.  These are initialized in parse_opt.  */

char *oname;			/* --output=FILE */
FILE *ofile;
char *input_directory;	/* --directory=DIR */
int  nlen_input_directory;
char *cmb_file_raw;	/* --cmb_file_raw=FILE */
int  nlen_cmb_file_raw;
char *cmb_file_smooth;	/* --cmb_file_smooth=FILE */
int  nlen_cmb_file_smooth;
char *cmb_file_points;	/* --cmb_file_points=FILE */
int  nlen_cmb_file_points;

int want_verbose;		/* --verbose */
int want_no_warn;		/* --no-warn */
int want_statistics_plot;            /* --statistics */
int want_circles_plot;            /* --circles */
int want_discs_plot;            /* --plot-discs */

#ifdef HAVE_ARGP_H
static struct argp_option options[] =
{
  { "output",      'o',           N_("FILE"),      0,
    N_("send Output to FILE instead of standard output"), 0 },
  { "circles",      'c',           NULL,      0,
    N_("plot Circles"), 0 },
  { "statistics",      's',           NULL,      0,
    N_("plot Statistics"), 0 },
  { "plot-discs",  'D',             NULL,       0,
    N_("plot Discs"), 0 }, 
  /*  { "angmax",  'a', N_("ANGMAX"),       0,
      N_("minimum Angular radius of circle"), 0 },
      { "angmax",  'b', N_("ANGMAX"),       0,
      N_("maximum angular radius of circle"), 0 }, 
  */
  { "verbose",     'v',           NULL,            0,
    N_("Print more information"), 0 },
  { "no-warn",     NOWARN_KEY,    NULL,            0,
    N_("disable Warnings"), 0 },
  { "directory",   DIRECTORY_KEY, N_("DIR"),       0,
    N_("Read input files from Directory DIR"), 0 },
  { "cmb_file_raw",  'i', N_("FILE"),       0,
    N_("cmb fits file of input data"), 0 },
  { "cmb_file_smooth",  'm', N_("FILE"),       0,
    N_("smoothed cmb file"), 0 },
  { "cmb_file_points",  'p', N_("FILE"),       0,
    N_("file with positions of point sources"), 0 },
  { NULL, 0, NULL, 0, NULL, 0 }
};

/* The argp functions examine these global variables.  */
const char *argp_program_bug_address = "<boud at astro.uni.torun.pl>";
void (*argp_program_version_hook) (FILE *, struct argp_state *) = show_version;

static struct argp argp =
{
  /*  options, parse_opt, N_("[FILE...]"), */
  options, parse_opt, N_(" "),
  N_("calculate identified circles statistics from CMB data (cosmic topology)"),
  NULL, NULL, NULL
};
#endif  /* HAVE_ARGP_H */


int
main (int argc, char **argv)
{

  char* result_getenv;  /* result of call to getenv - may be NULL */
  int   nlen_result_getenv;

  int want_debug = 1; 

  textdomain(PACKAGE);

  oname = "stdout";
  ofile = stdout;

  /* 
     INPUT PARAMETERS 
     certain input parameters can be input in many different ways 
  */

  /* firstly, try the header file */
#ifdef CIRCLES_INPUT_DIRECTORY
  input_directory= CIRCLES_INPUT_DIRECTORY;
  nlen_input_directory= strlen(input_directory);
  if(want_verbose)
    printf("CIRCLES_INPUT_DIRECTORY= %s\n", input_directory);
#else
  input_directory = NULL;
  nlen_input_directory= 0;
#endif
  /* secondly, try for environment variables */
#ifdef HAVE_STDLIB_H
  result_getenv= getenv("CIRCLES_INPUT_DIRECTORY");   
  if(result_getenv != NULL)
    {
      if(want_verbose)
	printf("CIRCLES_INPUT_DIRECTORY= %s\n", result_getenv);
      nlen_result_getenv = strlen(result_getenv);
      input_directory= xstrdup(result_getenv);
      nlen_input_directory= strlen(input_directory);
    };
#endif
  /* third try: command line option will override both the above */


#ifdef CIRCLES_CMB_FILE_RAW
  cmb_file_raw= CIRCLES_CMB_FILE_RAW;
  nlen_cmb_file_raw= strlen(cmb_file_raw);
  if(want_verbose)
    printf("CIRCLES_CMB_FILE_RAW= %s\n", cmb_file_raw);
#else
  cmb_file_raw = NULL;
  nlen_cmb_file_raw= 0;
#endif
  /* secondly, try for environment variables */
#ifdef HAVE_STDLIB_H
  result_getenv= getenv("CIRCLES_CMB_FILE_RAW");   
  if(result_getenv != NULL)
    {
      if(want_verbose)
	printf("CIRCLES_CMB_FILE_RAW= %s \n", result_getenv);
      nlen_result_getenv = strlen(result_getenv);
      cmb_file_raw= xstrdup(result_getenv);
      nlen_cmb_file_raw= strlen(cmb_file_raw);
    };
#endif
  /* third try: command line option will override both the above */


#ifdef CIRCLES_CMB_FILE_SMOOTH
  cmb_file_smooth= CIRCLES_CMB_FILE_SMOOTH;
  nlen_cmb_file_smooth= strlen(cmb_file_smooth);
  if(want_verbose)
    printf("CIRCLES_CMB_FILE_SMOOTH= %s\n", cmb_file_smooth);
#else
  cmb_file_smooth = NULL;
  nlen_cmb_file_smooth= 0;
#endif
  /* secondly, try for environment variables */
#ifdef HAVE_STDLIB_H
  result_getenv= getenv("CIRCLES_CMB_FILE_SMOOTH");   
  if(result_getenv != NULL)
    {
      if(want_verbose)
	printf("CIRCLES_CMB_FILE_SMOOTH= %s \n", result_getenv);
      nlen_result_getenv = strlen(result_getenv);
      cmb_file_smooth= xstrdup(result_getenv);
      nlen_cmb_file_smooth= strlen(cmb_file_smooth);
    };
#endif
  /* third try: command line option will override both the above */

#ifdef CIRCLES_CMB_FILE_POINTS
  cmb_file_points= CIRCLES_CMB_FILE_POINTS;
  nlen_cmb_file_points= strlen(cmb_file_points);
  if(want_verbose)
    printf("CIRCLES_CMB_FILE_POINTS= %s\n", cmb_file_points);
#else
  cmb_file_points = NULL;
  nlen_cmb_file_points= 0;
#endif
  /* secondly, try for environment variables */
#ifdef HAVE_STDLIB_H
  result_getenv= getenv("CIRCLES_CMB_FILE_POINTS");   
  if(result_getenv != NULL)
    {
      if(want_verbose)
	printf("CIRCLES_CMB_FILE_POINTS= %s \n", result_getenv);
      nlen_result_getenv = strlen(result_getenv);
      cmb_file_points= xstrdup(result_getenv);
      nlen_cmb_file_points= strlen(cmb_file_points);
    };
#endif
  /* third try: command line option will override both the above */


  want_verbose = 0;
  want_no_warn = 0;
  want_statistics_plot = 0;
  want_circles_plot = 0;
  want_discs_plot = 0;

#if HAVE_ARGP_H
  argp_parse(&argp, argc, argv, 0, NULL, NULL);
#else
  printf("Warning: no command line options since a recent glibc is not installed.\n");
#endif  /* HAVE_ARGP_H */

  /* simulate low RAM conditions */
  
  /* 
  printf("will try allocating another 10Mb... \n");
  xmalloc(10000000);
  */


  /* TODO: do the work */

  /* try to allocate memory for the big arrays, get back number of pixels */
  /*  PXALLO(want_verbose, &circles_npix); */


  pxalloc_status = -1;
  pxtempcp_status = -1;

  PXALLO(&want_verbose, &circles_npix,
	 &temp,&errtemp,&gallong,&gallat, &tempcp);

  printf("after pxallo: circles_npix = %d\n",circles_npix);

  if(want_debug)
    {
      printf("temp = %x , *temp = %x , *temp = %f\n",temp,*temp,*temp);
      printf("temp = %x , *temp = %x , *temp = %f\n",temp,*temp,*temp);
      printf("circles.c main: before   *temp= 7.8  value of *temp = %f;\n",*temp);
      *temp= 7.8;
      printf("circles.c main: after   *temp= 7.8  value of *temp = %f;\n",*temp);
      printf("temp = %x , *temp = %x, %f\n",temp,*temp,*temp);
      printf("temp = %x , *temp = %f\n",temp,*temp);
    };

  CIRC_F(
	 input_directory, &nlen_input_directory,
	 &circles_npix,
	 temp,errtemp,gallong,gallat,tempcp
	 ); 


  if(want_statistics_plot)
    {
      PDODEC();
    };

  if(want_circles_plot)
    {
      PCIRCLES(
	       input_directory, &nlen_input_directory
	       );
    };

  if(want_discs_plot)
    {
      PDISCS(
	     input_directory, &nlen_input_directory,
	     &circles_npix,
	     temp,errtemp,gallong,gallat,tempcp
	     );
    };


  exit (0);
}


#ifdef HAVE_ARGP_H
/* Parse a single option.  */
static error_t
parse_opt (int key, char *arg, struct argp_state *state)
{
  switch (key)
    {
    case ARGP_KEY_INIT:
      /* Set up default values.  */
      break;
    case 'o':			/* --output */
      oname = xstrdup (arg);
      ofile = fopen (oname, "w");
      if (!ofile)
	argp_failure (state, EXIT_FAILURE, errno,
		      _("Cannot open %s for writing"), oname);
      break;
    case 'c':
      want_circles_plot = 1;  /* --circles */
      break;
    case 's':
      want_statistics_plot = 1;  /* --statistics */
      break;
    case 'D':
      want_discs_plot = 1;  /* --plot-discs */
      break;
    case 'v':			/* --verbose */
      want_verbose = 1;
      break;
    case NOWARN_KEY:		/* --no-warn */
      want_no_warn = 1;
      break;
    case DIRECTORY_KEY:		/* --directory */
      /*      input_directory = xstrdup (optarg); */
      input_directory = xstrdup (arg);
      nlen_input_directory= strlen(input_directory);
      break;
    case 'i':	
      cmb_file_raw= xstrdup(arg);
      nlen_cmb_file_raw= strlen(cmb_file_raw);
      break;
    case 'm':	
      cmb_file_smooth= xstrdup(arg);
      nlen_cmb_file_smooth= strlen(cmb_file_smooth);
      break;
    case 'p':	
      cmb_file_points= xstrdup(arg);
      nlen_cmb_file_points= strlen(cmb_file_points);
      break;
    case ARGP_KEY_ARG:		/* [FILE]... */
      /* TODO: Do something with ARG, or remove this case and make
         main give argp_parse a non-NULL fifth argument.  */
      break;

    default:
      return ARGP_ERR_UNKNOWN;
    }
  return 0;
}
#endif  /* HAVE_ARGP_H */


/* Show the version number and copyright information.  */
static void
#ifdef HAVE_ARGP_H
show_version (FILE *stream, struct argp_state *state)
#else
     show_version (FILE *stream)
#endif  /* HAVE_ARGP_H */
{
#ifdef HAVE_ARGP_H
  (void) state;
#endif  /* HAVE_ARGP_H */
  /* Print in small parts whose localizations can hopefully be copied
     from other programs.  */
  fputs(PACKAGE" "VERSION"\n", stream);
  fprintf(stream, _("Written by %s.\n\n"), "Boud Roukema");
  fprintf(stream, _("Copyright (C) %s %s\n"), "2004", "Boud Roukema");
  fputs(_("\
This program is free software; you may redistribute it under the terms of\n\
the GNU General Public License.  This program has absolutely no warranty.\n"),
	stream);
}
